module.exports = {
    Panorama: {
        // Global variables to store configuration
        iFrameRequestUrl: null,
        panoramaContainer: null,
        iFrameElement: null,
        panoramaClickCheckBox: null,
        closeButton: null,
        keyboardMovementEnabled: null,
        mapClickEnabled: null,
        markerImage: null,
        // Variable to store whether the panorama is loaded or not
        panoramaLoaded: false,
        // Opens the panorama at the location x,y
        OpenPanoramaOnLocation: function (x, y) {
            // If panorama is closed, open it
            if (this.panoramaContainer.style.display == 'none') {
                // Open panorama
                this.OpenPanoramaFrame();
            }
            // If panorama is not loaded yet
            if (this.panoramaLoaded == false) {
                // Repeat this function until panorama is loaded
                const panorama = this;
                timer = setTimeout(function () {
                    panorama.OpenPanoramaOnLocation(x, y);
                }, 50);
            }
            // If panorama is loaded
            else {
                // If keyboard movement is enabled
                if (this.keyboardMovementEnabled == true) {
                    // Send this message once
                    this.keyboardMovementEnabled = false;
                    this.iFrameElement.contentWindow.postMessage({ keyboardEnabled: true }, '*');
                }
                // When the frame is loaded post message to open new coordinates
                this.iFrameElement.contentWindow.postMessage({ coordinates: [x, y] }, '*');
            }
        },
        // Opens the panorama iframe
        OpenPanoramaFrame: function () {
            // Fill in the iframes source
            if (this.iFrameElement.src != this.iFrameRequestUrl) {
                this.iFrameElement.src = this.iFrameRequestUrl;
            }
            // Unhide it
            this.panoramaContainer.style.display = 'initial';
            // Add this to close button and add an event listener
            this.closeButton.panorama = this;
            this.closeButton.addEventListener('click', this.HidePanoramaFrame);
            // Get the marker
            const marker = document.getElementById('panorama-view-cone');
            // If there is a marker
            if (marker != null) {
                // Enable marker
                marker.style.display = 'initial';
            }
        },
        // Hides the panorama frame
        HidePanoramaFrame: function (event) {
            // Hide container
            event.currentTarget.panorama.panoramaContainer.style.display = 'none';
            // Get the marker
            const marker = document.getElementById('panorama-view-cone');
            // If there is a marker
            if (marker != null) {
                // Remove marker
                marker.style.display = 'none';
            }
        },
        // Configure MapLibre map for panorama usage, call this method in map.onload
        ConfigureMapLibreMap: function (map, maplibregl) {
            // If keyboard movement is on
            if (this.keyboardMovementEnabled == true) {
                // Disable map keyboard controls
                map.keyboard.disable();
            }
            // Store this before events
            const panorama = this;
            // If map is clickable
            if (this.mapClickEnabled != false) {
                // Open panorama on click location
                map.on('click', function (e) {
                    // If checkbox is checked
                    if (panorama.panoramaClickCheckBox.checked == true) {
                        // Open panorama on location
                        panorama.OpenPanoramaOnLocation(e.lngLat.lng, e.lngLat.lat);
                    }
                })
            }
            // If marker is enabled (exists)
            if (panorama.markerImage != null) {
                // Create new marker
                const newMarker = document.createElement('div');
                newMarker.id = 'panorama-view-cone';
                const img = new Image();
                img.addEventListener("load", function () {
                    newMarker.style.width = this.naturalWidth + 'px';
                    newMarker.style.height = this.naturalHeight + 'px';
                    newMarker.style.backgroundImage = 'url(' + this.src + ')';
                });
                img.src = panorama.markerImage;
                //newMarker.style.backgroundImage = "url(" + panorama.markerImage + ")";
                let maplibreMarker = new maplibregl.Marker(newMarker);
                // When new location or angle comes as message
                window.addEventListener('message', function (event) {
                    if (event.data.angle != undefined && event.data.coordinates != undefined) {
                        // If panorama is open
                        if (panorama.panoramaContainer.style.display != 'none') {
                            // Get the marker
                            const marker = document.getElementById('panorama-view-cone');
                            // If there is a marker
                            if (marker != null) {
                                // Get its coordinates
                                const markerCoordinates = maplibreMarker.getLngLat();
                                // If the new coordinates are different than the markers coordinates
                                if (markerCoordinates != null && markerCoordinates.lng != parseFloat(event.data.coordinates[0]) && markerCoordinates.lat != parseFloat(event.data.coordinates[1])) {
                                    // Center map
                                    map.flyTo({ center: event.data.coordinates });
                                }
                            }
                            else {
                                maplibreMarker.setLngLat([29,41]).addTo(map);
                            }
                            maplibreMarker.setLngLat(event.data.coordinates).setRotation(event.data.angle);
                        }
                    }
                })
            }
        },
        // Configure Panorama iFrameId is html id of the iframe, close button id is the html id of the close button, if marker is rendered on map markerId is the html id of the marker
        ConfigurePanorama: function (properties) {
            // Initialize global variables
            this.iFrameRequestUrl = properties.iFrameElementSrc;
            // Optional, default is panorama-container
            if (properties.panoramaContainerId != null) {
                this.panoramaContainer = document.getElementById(properties.panoramaContainerId);
            }
            else {
                this.panoramaContainer = document.getElementById('panorama-container');
            }
            // Optional, default is panorama-iframe
            if (properties.panoramaIframeId != null) {
                this.iFrameElement = document.getElementById(properties.panoramaIframeId);
            }
            else {
                this.iFrameElement = document.getElementById('panorama-iframe');
            }
            // Optional, default is panorama-close-button
            if (properties.panoramaCloseButtonId != null) {
                this.closeButton = document.getElementById(properties.panoramaCloseButtonId);
            }
            else {
                this.closeButton = document.getElementById('panorama-close-button');
            }
            // Optional, default is false
            if (properties.mapClickEnabled != null) {
                this.mapClickEnabled = properties.mapClickEnabled;
                this.markerImage = 'https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png';
                this.panoramaClickCheckBox = document.getElementById('panorama-checkbox');
            }
            else {
                this.mapClickEnabled = false;
            }
            // Optional, default is none element, false mapclick
            if (properties.panoramaCheckBoxElementId != null) {
                this.panoramaClickCheckBox = document.getElementById(properties.panoramaCheckBoxElementId);
                this.mapClickEnabled = true;
            }
            // Optional, default is 'https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png'
            if (properties.markerImage != null) {
                this.markerImage = properties.markerImage;
            }
            // Optional, default is true
            if (this.keyboardMovementEnabled != null) {
                this.keyboardMovementEnabled = properties.keyboardControlsEnabled;
            }
            else {
                this.keyboardMovementEnabled = true;
            }
            // Store this before events
            const panorama = this;
            // Catch the panorama loaded message
            window.addEventListener('message', function (event) {
                if (event.data.load == true) {
                    panorama.panoramaLoaded = true;
                }
            });
            // If configuration allows keyboard movement
            if (this.keyboardMovementEnabled == true) {
                panorama.iFrameElement.contentWindow.postMessage({ keyboardEnabled: true }, '*');
            }
        }
    }
}